if(typeof Dropzone != 'undefined') {
	Dropzone.autoDiscover = false;
}
+function($) {
	'use strict';

	/**
	 * Restrict
	 *
	 * Lib to restrict text box input
	 */

	$.fn.fionaRestrict = function() {
		return this.each(function() {
			var $this = $(this);
			var allowedChars = $this.data('allow-chars').split(' ');
			var whitelist = [];

			//Command Keys
			whitelist.push('Escape','Esc','Tab','CapsLock','Shift','Control','Alt','Meta','Backspace','Enter','ArrowUp','ArrowRight','ArrowDown','ArrowLeft','PageUp','PageDown','Home','End','Insert','Delete','Del','F1','F2','F3','F4','F5','F6','F7','F8','F9','F10','F11','F12');

			if(allowedChars.indexOf('numeric') >= 0) {
				var numeric = '1234567890';
				numeric = numeric.split('');
				whitelist = whitelist.concat(numeric);
				allowedChars.splice(allowedChars.indexOf('numeric'),1);
			}

			if(allowedChars.indexOf('symbols') >= 0) {
				var symbols = '`~!@#$%^&*()-_=+[{]}\\|;:\'",<.>/?';
				symbols = symbols.split('');
				whitelist = whitelist.concat(symbols);
				allowedChars.splice(allowedChars.indexOf('symbols'),1);
			}

			if(allowedChars.indexOf('alpha') >= 0) {
				var alpha = 'abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ';
				alpha = alpha.split('');
				whitelist = whitelist.concat(alpha);
				allowedChars.splice(allowedChars.indexOf('alpha'),1);
			}

			if(allowedChars.indexOf('alphaLower') >= 0) {
				var alpha = 'abcdefghijklmnopqrstuvwxyz';
				alpha = alpha.split('');
				whitelist = whitelist.concat(alpha);
				allowedChars.splice(allowedChars.indexOf('alphaLower'),1);
			}

			if(allowedChars.indexOf('alphaUpper') >= 0) {
				var alpha = 'ABCDEFGHIJKLMNOPQRSTUVWXYZ';
				alpha = alpha.split('');
				whitelist = whitelist.concat(alpha);
				allowedChars.splice(allowedChars.indexOf('alphaUpper'),1);
			}

			if(allowedChars.indexOf('space') >= 0) {
				whitelist.push(' ','Space','Spacebar');
				allowedChars.splice(allowedChars.indexOf('space'),1);
			}

			whitelist = whitelist.concat(allowedChars);

			$this.on({
				keypress: function(event) {
					var input = event.key || event.charCode;
					input = input === parseInt(input) ? String.fromCharCode(input) : input;

					if(whitelist.indexOf(input) < 0) {
						event.preventDefault();
						return;
					}
				},
				paste: function(event) {
					var pasteContent = (event.originalEvent || event).clipboardData.getData('text/plain') || prompt('Paste something..');
					
					var cursorStart = this.selectionStart;
					var cursorEnd = this.selectionEnd;

					var initValue = $this.val();
					var initValueBefore = initValue.substr(0,cursorStart);
					var initValueAfter = initValue.substr(cursorEnd,initValue.length);

					var content = '';
					for(var i = 0; i < pasteContent.length; i++) {
						if(whitelist.indexOf(pasteContent.charAt(i)) >= 0) {
							content += pasteContent.charAt(i);
						}
					}

					var processedContent = initValueBefore + content + initValueAfter;

					$this.val(processedContent);
					this.selectionStart = cursorStart + content.length;
					this.selectionEnd = this.selectionStart;
					event.preventDefault();
					return;
				}
			});
		});
	}

}(jQuery);

+function($) {
	'use strict';

	/**
	 * Auto Thousand
	 *
	 * Automatically adds a thousand separator
	 */

	$.fn.fionaAutoThousand = function() {
		return this.each(function() {
			var $this = $(this);
			var initValue;
			var cursorStart;
			var whitelist = [];

			//Command Keys
			whitelist.push('Escape','Esc','Tab','CapsLock','Shift','Control','Alt','Meta','Enter','ArrowUp','ArrowRight','ArrowDown','ArrowLeft','PageUp','PageDown','Home','End','Insert','F1','F2','F3','F4','F5','F6','F7','F8','F9','F10','F11','F12');

			$this.on({
				keydown: function() {
					initValue = $this.val();
				},
				keyup: function(event) {
					var value = $this.val();
					if(whitelist.indexOf(event.key) < 0 && !event.ctrlKey && value != initValue) {
						var cursorStart = this.selectionStart;

						var valueLength = value.length;
						value = value.replace(/,/g,'');

						// result  = parseInt( value ).toLocaleString('en-US');
						var result = value.toString().replace(/\B(?=(\d{3})+(?!\d))/g, ",");
						var resultLength = result.length;

						$this.val(result);
						this.selectionStart = cursorStart + (resultLength - valueLength);
						this.selectionEnd = this.selectionStart;
					}
				},
				paste: function() {
					$this.trigger('keyup');
				}
			})
		});
	}

}(jQuery);

+function($) {
	'use strict';

	/**
	 * Password
	 *
	 * Reset password function
	 */

	$.fn.fionaPassword = function() {
		return this.each(function() {
			var $this = $(this);
			var data = {
				id: $this.data('id'),
				container: $this.data('container'),
				name: $this.data('name'),
				required: $this.data('required'),
				width: $this.data('width'),
				placeholder: $this.data('placeholder')
			};

			$this.on({
				click: function(event) {
					if(!$this.hasClass('is-reset')) {
						var input = '<div id="' + data.container + '"><input id="' + data.id + '" class="fiona-password form-control" ' + (data.width != '' ? 'style="width:' + data.width + ' !important;"' : '') + ' name="' + data.name + '::password' + (data.required != '' ? '::req' : '') + '" ' + (data.required != '' ? 'required' : '') + ' type="password" ' + (data.placeholder != '' ? 'placeholder="' + data.placeholder + '"' : '') + '><br></div>';
						$this.addClass('is-reset').html('Cancel').before(input);
					} else {
						$this.removeClass('is-reset').html('Reset Password');
						$('#' + data.container).remove();
					}
				}
			});
		});
	}

}(jQuery);

+function($) {
	'use strict';

	/**
	 * Text Editor
	 *
	 * Text Editor functions
	 */

	function recalcColor( begin, end, factor ) {
		var newRed = Math.floor(begin.red + ( ( end.red - begin.red ) * factor ));
		var newGreen = Math.floor(begin.green + ( ( end.green - begin.green ) * factor ));
		var newBlue = Math.floor(begin.blue + ( ( end.blue - begin.blue ) * factor ));
		return 'rgb(' + newRed + ',' + newGreen + ',' + newBlue + ')';
	}

	$.fn.fionaTextEditor = function() {
		return this.each(function() {
			var $this = $(this);
			var data = {
				id: $this.attr('id'),
				index: $this.data('index'),
				modal: $this.data('modal'),
				pluginsClass: $this.data('plugins-image-class'),
				pluginsDimension: $this.data('plugins-image-dimension'),
				width: $this.data('width'),
				height: $this.data('height'),
				resize: $this.data('resize')
			};
			var $formContainer = $('#form-' + data.index);
			var $modal = $('#' + data.modal);
			var $modal = {
				container: $modal,
				body: $modal.find('.modal-body'),
				media: $modal.find('.media-files'),
				recent: $modal.find('.recent-container'),
				select: $modal.find('.image-select')
			};
			var $modalData = {
				maxFilesize: $modal.container.data('max-filesize'),
				uploadPath: $modal.container.data('upload-path'),
				upload: $modal.container.data('id-upload'),
				preview: $modal.container.data('id-dz-preview'),
				template: $modal.container.data('id-dz-template')
			};
			var isLoaded = false;

			var tinymcePlugin = 'fionadrive_' + data.index;

			tinymce.PluginManager.add(tinymcePlugin, function(editor, url) {
				editor.addButton(tinymcePlugin, {
					icon: 'image',
					tooltip: 'Insert image',
					onclick: function() {
						$modal.container.modal('show');
						
						if(!isLoaded) {
							$.ajax({
								type: "POST",
								url: fionaUrl + '/system/ajax_drive_get_files',
								dataType: "text",
								timeout:15000
							}).done(function(ajaxData) {
								$modal.media.html(ajaxData);
								isLoaded = true;
							}).fail(function(jqxhr, textStatus, error) {
								alert('Time out :(');
								console.error("AJAX failed, status: " + textStatus + ", error: "+error);
							});
						}
					}
				});
			});

			tinymce.init({
				selector: "#" + data.id,
				theme: "modern",
				relative_urls: false,
				// skin: "fiona",
				width: data.width,
				height: data.height,
				resize: data.resize,
				fontsize_formats: "Default=inherit 8pt 10pt 12pt 14pt 18pt 24pt 36pt",
				menubar : false,
				
				plugins: [
					"" + tinymcePlugin + " advlist autolink code link image lists charmap hr anchor pagebreak",
					"searchreplace wordcount visualblocks visualchars insertdatetime media nonbreaking",
					"table directionality emoticons paste textcolor fullscreen"
				],
				
				toolbar: [
					"bold italic underline strikethrough | undo redo | alignleft aligncenter alignright alignjustify | blockquote hr | bullist numlist",
					"" + tinymcePlugin + " media | fontsizeselect forecolor backcolor | outdent indent | table | link unlink | removeformat | code fullscreen"
				],
				image_advtab: false ,
			});

			$formContainer.on('click','.image-select',function() {
				var $selectedImage = $(this);
				var imgUrl = $selectedImage.attr('data-url');
				var imgFileName = $selectedImage.attr('data-filename');

				var content = fionaSiteUrl + 'image/' + imgFileName + '?o=' + data.pluginsDimension;
				content = '<img class="' + data.pluginsClass + '" src="' + content + '" />';

				tinymce.get(data.id).execCommand('mceInsertContent', false, content);
				$modal.container.modal('hide');
			});

			$modal.body.scroll(function(){
				var $scrolledElement = $(this);
				$scrolledElement = {
					container: $scrolledElement,
					recent: $scrolledElement.find('.recent-container')
				};
				if($modal.body.hasClass('scroll-fx')) {
					var scrollTop = $scrolledElement.container.scrollTop();
					var innerHeight = $scrolledElement.recent.innerHeight() - 55;

					var backgroundColor = {red:245,green:245,blue:245};
					var borderColor = {red:224,green:224,blue:224};
					var endColor = {red:255,green:255,blue:255};

					if(scrollTop >= 0 && scrollTop <= innerHeight ) { 
						var percentScrolled = scrollTop / ( innerHeight - 0 );

						var newBackgroundColor = recalcColor(backgroundColor,endColor,percentScrolled);
						var newBorderColor = recalcColor(borderColor,endColor,percentScrolled);
						$modal.media.css({ backgroundColor: newBackgroundColor });
						$modal.media.css({ borderColor: newBorderColor });
					} else if ( scrollTop > innerHeight ) {
						$modal.media.css({ backgroundColor: 'rgb(' + endColor.red + ',' + endColor.green + ',' + endColor.blue + ')' });
						$modal.media.css({ borderColor: 'rgb(' + endColor.red + ',' + endColor.green + ',' + endColor.blue + ')' });
					} else if ( scrollTop < 0 ) {
						$modal.media.css({ backgroundColor: 'rgb(' + backgroundColor.red + ',' + backgroundColor.green + ',' + backgroundColor.blue + ')' });
						$modal.media.css({ borderColor: 'rgb(' + borderColor.red + ',' + borderColor.green + ',' + borderColor.blue + ')' });
					}
				}
			});

			var recentFiles = new Array;
			$modal.container.dropzone({
				url: fionaUrl + '/system/ajax_drive_modal_upload',
				maxFilesize: $modalData.maxFilesize,
				acceptedFiles:'image/*',
				// uploadMultiple:true,
				// addRemoveLinks:true,
				previewTemplate: $('#' + $modalData.template).html(),
				previewsContainer: "#" + $modalData.preview,
				clickable: "#" + $modalData.upload,
				dictDefaultMessage:"",

				init:function() {
					this.on("addedfile", function(file) {
						$modal.recent.addClass('recent-exist');
						$modal.media.addClass('recent-exist');
						$modal.body.removeClass('recent-invisible');
					});
					this.on("removedfile", function(file) {
					});
					this.on("thumbnail", function(file, dataUrl){
					});
					this.on("processing", function(file){
					});
					this.on("uploadprogress", function(file, progress){
					});
					this.on("sending", function(file){
					});
					this.on("success", function(file, uploadedFileName){
						var fileUrl = $modalData.uploadPath;
						var fileName = uploadedFileName;
						var fileSize = file.size;
						var fileMime = file.type;
						recentFiles.push(
							{file_url:fileUrl,file_name:fileName,file_mime:fileMime,file_size:fileSize}
						);
						
						var imageSelect = $(file.previewElement);
						imageSelect.attr('data-url',fileUrl);
						imageSelect.attr('data-filename',fileName);

						if($modal.select.length == 1) {
							$modal.select.css({ opacity: 0 });
							setTimeout(function(){
								$modal.body.addClass('scroll-fx');
								file.previewElement.querySelector("img").src = fionaSiteUrl + '/image/' + fileName + '?o=h150';
								$modal.select.css({ opacity: 1 });
							}, 1000);
						} else file.previewElement.querySelector("img").src = fionaSiteUrl + '/image/' + fileName + '?o=h150';
					});
					this.on("error", function(file){
					});
					this.on("complete", function(file){
					});
					this.on("canceled", function(file){
					});
					this.on("queuecomplete", function(){
						$.ajax({
							type: "POST",
							url: fionaUrl + '/system/ajax_drive_modal_save_db',
							data: {files:recentFiles},
							dataType: "text"
						}).done(function(data) {
							recentFiles = new Array;
						}).fail(function(jqxhr, textStatus, error) {
							console.error("AJAX failed, status: " + textStatus + ", error: "+error);
						});
					});
					this.on("maxfilesexceeded", function(file){
					});
				}
			});

		});
	}

}(jQuery);

+function($) {
	'use strict';

	/**
	 * File
	 *
	 * File functions
	 */

	$.fn.fionaFile = function() {
		return this.each(function() {
			var $this = $(this);
			var data = {
				values: $this.data('values'),
				acceptedFiles: $this.data('accepted-files'),
				multiple: $this.data('multiple'),
				uniqid: $this.data('unique-id'),
				maxFilesize: $this.data('max-filesize'),
				$input: $('#' + $this.data('id-input')),
				uniqidInput: $this.data('unique-id-input'),
				upload: $this.data('id-upload'),
				preview: $this.data('id-dz-preview'),
				template: $this.data('id-dz-template')
			};
			var maxFiles = 1;
			if(data.multiple) {
				maxFiles = null;
			}

			$this.dropzone({
				url: fionaUrl + '/system/ajax_file_upload/' + data.uniqid,
				maxFilesize: data.maxFilesize,
				createImageThumbnails: false,
				maxFiles: maxFiles,
				acceptedFiles: data.acceptedFiles,
				// addRemoveLinks:true,
				previewTemplate: $('#' + data.template).html(),
				previewsContainer: '#' + data.preview,
				clickable: '#' + data.upload,
				dictDefaultMessage:"", 

				init:function() {
					this.on("addedfile", function(file) {
						$this.addClass('dropzone-file-uploaded');
						if(data.values !== null && !data.multiple) {
							data.$input.val('');
							$('#' + data.preview + ' .current').remove();
						}
					});

					this.on("removedfile", function(file) {
						if($this.find('.dropzone-preview-item').length == 0) {
							$this.removeClass('dropzone-file-uploaded');
							$this.removeClass('dropzone-success');
							$this.removeClass('dropzone-error');
							$this.find('.dropzone-error-message').html('');
						}
						if(file.width == undefined || file.width == null) file.width = 0;
						if(file.height == undefined || file.height == null) file.height = 0;
						var fileInput = file.name + ':' + file.type + ':' + file.width + ':' + file.height + ':' + file.size;
						var dropzoneInput = data.$input.val();
						dropzoneInput = dropzoneInput.replace(fileInput + (data.multiple ? ';' : ''),'');
						data.$input.val(dropzoneInput);

						$.ajax({
							type: "POST",
							url: fionaUrl + '/system/ajax_file_remove/' + data.uniqid,
							data: {file_name:file.name},
							dataType: "text",
							timeout:15000
						}).done(function( data ) {
						}).fail(function(jqxhr, textStatus, error) {
							console.error("AJAX failed, status: " + textStatus + ", error: "+error);
						});
					});

					this.on("thumbnail", function(file, dataUrl){
					});
					this.on("processing", function(file){
					});
					this.on("uploadprogress", function(file, progress){
					});
					this.on("sending", function(file){
					});
					this.on("success", function(file){
						$.ajax({
							type: "POST",
							url: fionaUrl + '/system/ajax_file_thumbnail/' + data.uniqid,
							data: {file_name:file.name,file_mime:file.type},
							dataType: "text",
							timeout:15000
						}).done(function( ajaxData ) {
							// console.log($(file.previewElement).find());
							$this.addClass('dropzone-success');
							if(file.width == undefined || file.width == null) file.width = 0;
							if(file.height == undefined || file.height == null) file.height = 0;
							var fileInput = file.name + ':' + file.type + ':' + file.width + ':' + file.height + ':' + file.size;
							var dropzoneInput = data.$input.val();
							data.$input.val(dropzoneInput + fileInput + (data.multiple ? ';' : ''));

							// console.log($(file.previewElement));
							// file.previewElement.querySelector("img").src = data;
							$(file.previewElement).children('.dropzone-preview-thumbnail-container').html(ajaxData);
							$(file.previewElement).children('.dropzone-remove').children('.dropzone-remove-label').html('Unselect<br><code>' + file.name + '</code>');
						}).fail(function(jqxhr, textStatus, error) {
							console.error("AJAX failed, status: " + textStatus + ", error: "+error);
						});
					});
					this.on("error", function(file){
						$this.addClass('dropzone-error');
					});
					this.on("complete", function(file){
					});
					this.on("canceled", function(file){
					});
					this.on("queuecomplete", function(file){
					});
					this.on("maxfilesexceeded", function(file){
						this.removeAllFiles(true);
						this.addFile(file);
					});

					this.on("dragover", function(event){
						$this.addClass('dropzone-hover');
					});
					this.on("dragleave", function(event){
						$this.removeClass('dropzone-hover');
					});
					this.on("drop", function(event){
						$this.removeClass('dropzone-hover');
					});
				}
			});

			$this.find('.dropzone-preview-container').scroll(function(){
			    var scrollTop = $(this).scrollTop();
			    if (scrollTop <= 10) {
			    	$this.find('.dropzone-area').removeClass('cast-shadow');
			    } else {
			    	$this.find('.dropzone-area').addClass('cast-shadow');
			    }
			});

			$('#' + data.preview).on('click','.dropzone-remove',function(event) {
				var fileInput = $(this).data('value');
				var dropzoneInput = data.$input.val();
				dropzoneInput = dropzoneInput.replace(fileInput + (data.multiple ? ';' : ''),'');
				data.$input.val(dropzoneInput);

				$(this).parent().remove();
				if($this.find('.dropzone-preview-item').length == 0) {
					$this.removeClass('dropzone-file-uploaded');
					$this.removeClass('dropzone-success');
					$this.removeClass('dropzone-error');
					$this.find('.dropzone-error-message').html('');
				}
			});
		});
	}

}(jQuery);

+function($) {
	'use strict';

	/**
	 * Date
	 *
	 * Date functions
	 */

	$.fn.fionaDate = function() {
		return this.each(function() {
			var $this = $(this);
			var data = {
				id: $this.attr('id'),
				type: $this.data('type')
			};

			$this.datepicker({
				todayBtn: "linked",
				autoclose: true,
				todayHighlight: true
			});

			if(data.type == 'range') {
				var $dateStart = $('#' + data.id + '-start');
				var $dateEnd = $('#' + data.id + '-end');

				$dateStart.datepicker().on({
					changeDate: function(event) {
						$dateEnd.focus();
					}
				});
			}

		});
	}

}(jQuery);

+function($) {
	'use strict';

	/**
	 * Switch
	 *
	 * Switch functions
	 */

	$.fn.fionaSwitch = function() {
		return this.each(function() {
			var $this         = $(this);
			var on            = $this.data('on');
			var off           = $this.data('off');
			
			var $switch       = $this.attr('id') + '-switch';
			$switch           = $('#' + $switch);
			var $switchOn     = $switch.children('.switch-on');
			var $switchOff    = $switch.children('.switch-off');
			var $switchSlider = $switch.children('.switch-slider');

			$switchOn.on({
				click: function(event) {
					$switch.removeClass('off');
					$this.attr('value',on);
				}
			});

			$switchOff.on({
				click: function(event) {
					$switch.addClass('off');
					$this.attr('value',off);
				}
			});

			$switchSlider.on({
				click: function(event) {
					$switch.toggleClass('off');
					if($switch.hasClass('off')) {
						$this.attr('value',off);
					} else {
						$this.attr('value',on);
					}
				}
			});
		});
	}

}(jQuery);

+function($) {
	'use strict';

	/**
	 * Map
	 *
	 * Map functions
	 */

	function createMarker(map,markers,latlng) {
		markers = deleteMarkers(markers);
		var marker = new google.maps.Marker({
			map: map,
			position: latlng
		});
		markers = setAllMap(map,markers);
		markers.push(marker);
		return markers;
	}
	
	function setAllMap(map,markers) {
		for (var i = 0; i < markers.length; i++) {
			markers[i].setMap(map);
		}
		return markers;
	}
	
	function deleteMarkers(markers) {
		markers = setAllMap(null,markers);
		markers = [];
		return markers;
	}

	$.fn.fionaMap = function() {
		return this.each(function() {
			var $this = $(this);
			var data = {
				index: $this.data('index'),
				id: $this.data('id'),
				$lat: $('#' + $this.data('lat-id')),
				$lng: $('#' + $this.data('lng-id')),
				$modal: $('#' + $this.data('modal-id')),
				map: $this.data('map-id'),
				mapSearch: $this.data('map-search-id'),
				values: $this.data('values'),
				lat: $this.data('lat'),
				lng: $this.data('lng')
			};
			var init = {
				lat: $this.data('init-lat'),
				lng: $this.data('init-lng'),
				z: $this.data('init-zoom')
			};

			var map;
			var input;
			var searchBox;
			var places;
			var markers = [];
			var start = false;
			var lat;
			var lng;
			var initLat = data.lat;
			var initLng = data.lng;
			data.$modal.on('shown.bs.modal',function() {
				if(!start) {
					start = true;
					var latlng = new google.maps.LatLng(init.lat, init.lng);
					var styles = [ 
								  ];
					var myOptions = {
						center: latlng,
						zoom: init.z,
						streetViewControl:false,
						draggableCursor: 'crosshair',
						mapTypeId: google.maps.MapTypeId.ROADMAP,
						styles: styles
					};
					
					map = new google.maps.Map(document.getElementById(data.map), myOptions);

					input = document.getElementById(data.mapSearch);
					searchBox = new google.maps.places.SearchBox(input);
					map.controls[google.maps.ControlPosition.TOP_LEFT].push(input);

					// Bias the SearchBox results towards current map's viewport.
					map.addListener('bounds_changed', function() {
						searchBox.setBounds(map.getBounds());
					});

					searchBox.addListener('places_changed', function() {
						places = searchBox.getPlaces();

						if (places.length == 0) {
							return;
						}

						// Clear out the old markers.
						markers.forEach(function(marker) {
							marker.setMap(null);
						});
						markers = [];

						// For each place, get the icon, name and location.
						var bounds = new google.maps.LatLngBounds();
						places.forEach(function(place) {

							lat = place.geometry.location.lat();
							lng = place.geometry.location.lng();

							// Create a marker for each place.
							markers.push(new google.maps.Marker({
								map: map,
								title: place.name,
								position: place.geometry.location
							}));

							if (place.geometry.viewport) {
								// Only geocodes have viewport.
								bounds.union(place.geometry.viewport);
							} else {
								bounds.extend(place.geometry.location);
							}
						});
						map.fitBounds(bounds);
						if(map.getZoom() > 17) {
							map.setZoom(17);
						}
					});

					if(data.values == 1) {
						var initLatlng = new google.maps.LatLng(data.$lat.val(), data.$lng.val());
						markers = createMarker(map,markers,initLatlng);
					}
					

					google.maps.event.addListener(map, 'click', function(event) {
						lat = event.latLng.lat();
						lng = event.latLng.lng();

						var latlng = new google.maps.LatLng(lat, lng);
						
						markers = createMarker(map,markers,latlng);

						var geocoder = new google.maps.Geocoder();
						geocoder.geocode({
							latLng: latlng
						}, function(responses) {
							if (responses && responses.length > 0) {
								document.getElementById(data.mapSearch).value = responses[0].formatted_address;
							} else {
								document.getElementById(data.mapSearch).value = 'Cannot determine address at this location.';
							}
						});
					});

					setTimeout(function() {
						$('#' + data.mapSearch).addClass('show');
						$('#' + data.mapSearch).focus();
					},1000);
				} else {
					if(initLat != '') map.panTo(new google.maps.LatLng(initLat, initLng));
				}
			});

			data.$modal.on('hidden.bs.modal',function() {
				if(initLat != '') {
					$('#form-' + data.index + ' .map-btn').html('Change Location');
				} else {
					$('.map-latlng').removeClass('location-set');
				}
				var initLatlng = new google.maps.LatLng(initLat, initLng);
				data.$lat.val(initLat);
				data.$lng.val(initLng);
				markers = createMarker(map,markers,initLatlng);
			});

			data.$modal.on('click','.modal-yes',function() {
				$('#form-' + data.index + ' .map-btn').html('Change Location');
				$('#' + data.id).val($('#' + data.mapSearch).val());
				data.$lat.val(lat);
				data.$lng.val(lng);
				$('.map-latlng').addClass('location-set');
				initLat = lat;
				initLng = lng;
			});
		});
	}

}(jQuery);

+function($) {
	'use strict';

	/**
	 * Tabs
	 *
	 * Tabs functions
	 */

	$.fn.fionaTabs = function() {
		return this.each(function() {
			var $this = $(this);

			$this.on({
				click: function() {
					var element = $(this);
					element.data = {
						parent : element.parents('.form-tabs'),
						tab : element.data('tab')
					}

					element.data.parent.find('.form-tabs-header-item').removeClass('active');
					element.addClass('active');

					element.data.parent.find('.form-tabs-item').removeClass('active');
					element.data.parent.find('.form-tabs-item[data-tab="' + element.data.tab + '"]').addClass('active');

					// autosize.update(element.data.parent.find('.form-tabs-item[data-tab="' + element.data.tab + '"] textarea'));
				}
			});
		});
	}

}(jQuery);

+function($) {
	'use strict';

	/**
	 * Max Length
	 * 
	 * Gives a maximum length of an input with a visual feedback
	 */
	
	function applyStyle($indicator,lengthLeft,maxLength) {
		if(lengthLeft >= maxLength / 2) {
			$indicator.removeClass('text-secondary');
			$indicator.removeClass('text-tertiary');
		}

		if(lengthLeft < maxLength / 2 && lengthLeft > 0) {
			$indicator.removeClass('text-tertiary');
			$indicator.addClass('text-secondary');
		}

		if(lengthLeft == 0) {
			$indicator.removeClass('text-secondary');
			$indicator.addClass('text-tertiary');
		}
	}

	$.fn.fionaMaxLength = function() {
		return this.each(function() {
			var $this            = $(this);
			
			//Whitelist of Command Keys
			var whitelist        = [];
			whitelist.push('Escape','Esc','Tab','CapsLock','Shift','Control','Alt','Meta','Backspace','Enter','ArrowUp','ArrowRight','ArrowDown','ArrowLeft','PageUp','PageDown','Home','End','Insert','Delete','Del','F1','F2','F3','F4','F5','F6','F7','F8','F9','F10','F11','F12');
			
			var id               = $this.attr('id') + '-maxlength';
			var maxLength        = $this.data('max-length');
			$this.val($this.val().substr(0,maxLength));
			
			//Create Indicator
			var initIndicator    = maxLength - $this.val().length;
			$this.after('<div id ="' + id + '" class="fiona-maxlength-indicator"></div>');
			var $indicator       = $('#' + id);
			$indicator.html(initIndicator);
			applyStyle($indicator,initIndicator,maxLength);

			$this.on({
				keypress: function(event) {
					var input = event.key || event.charCode;
					input = input === parseInt(input) ? String.fromCharCode(input) : input;

					if(whitelist.indexOf(input) < 0) {
						var value = $this.val().length;
						if(value + 1 > maxLength) {
							event.preventDefault();
							return;
						}
					}
				},
				keyup: function(event) {
					var value = $this.val().length;
					var lengthLeft = maxLength - value;

					applyStyle($indicator,lengthLeft,maxLength);

					$indicator.html(lengthLeft);
				},
				paste: function(event) {
					var pasteContent = (event.originalEvent || event).clipboardData.getData('text/plain') || prompt('Paste something..');
					
					var cursorStart = this.selectionStart;
					var cursorEnd = this.selectionEnd;
					var cursorRange = cursorEnd - cursorStart;

					var initValue = $this.val();
					var initValueBefore = initValue.substr(0,cursorStart);
					var initValueAfter = initValue.substr(cursorEnd,initValue.length);
					var lengthLeft = maxLength - initValue.length + cursorRange;

					var content = pasteContent.substr(0,lengthLeft);

					var processedContent = initValueBefore + content + initValueAfter;

					$this.val(processedContent);
					this.selectionStart = cursorStart + content.length;
					this.selectionEnd = this.selectionStart;
					event.preventDefault();
					return;
				}
			})
		});
	}

}(jQuery);

+function($) {
	'use strict';

	/**
	 * Data Table
	 *
	 * Data Table functions
	 */

	$.fn.fionaDataTable = function() {
		return this.each(function() {
			var $this = $(this);
			var $checkboxHeader = $('.checkbox-header');

			var control = {
				$container: $('.data-management'),
				$remove: $('.remove-selected'),
				$count: $('.count-selected'),
				$delete: $('.delete-selected'),
				$filter: $('#word-filter')
			};

			var lang = {
				first: $this.data('lang-first'),
				previous: $this.data('lang-previous'),
				next: $this.data('lang-next'),
				last: $this.data('lang-last'),
				pagination: $this.data('lang-pagination'),
				filtered: $this.data('lang-filtered'),
				empty: $this.data('lang-empty'),
				emptyFiltered: $this.data('lang-empty-filtered')
			};

			var dataTableOption = {
				orderMulti: false,
				orderFixed: [2,'desc'],
				columnDefs: [
					{targets:'checkbox-header',orderable:false},
				],
				scrollY:'calc(100vh - 200px)',
				pagingType: 'full_numbers', 
				pageLength: 10, 
				info:true,

				language: {
					paginate: {
						first: lang.first,
						previous: lang.previous,
						next: lang.next,
						last: lang.last
					},
					info: lang.pagination,
					infoFiltered: lang.filtered,
					emptyTable: lang.empty,
					zeroRecords: lang.emptyFiltered
				}
			};

			var sort = $this.data('sort');
			var sortChanges;
			if(sort !== false) {
				dataTableOption.order = [0,'asc'];
				dataTableOption.rowReorder = {
					snapX: 25
				};
				dataTableOption.columnDefs.push({targets:'_all',orderable:false});
			} else {
				dataTableOption.order = [];
				dataTableOption.ordering = true;
			}

			var $dataTable = $this.DataTable(dataTableOption);
			$dataTable.on('select',function() {
				var count = $dataTable.rows( { selected: true } ).count();
				var countFalse = $dataTable.rows( { selected: false } ).count();
				control.$count.text(count + ' item' + (count > 1 ? 's' : '') + ' selected');
				control.$container.addClass('multi-select');
				if(countFalse == 0) {
					$checkboxHeader.removeClass('select-some');
					$checkboxHeader.addClass('select-all');
				} else {
					$checkboxHeader.removeClass('select-all');
					$checkboxHeader.addClass('select-some');
				}
			});

			$dataTable.on('deselect',function() {
				var count = $dataTable.rows( { selected: true } ).count();
				var countFalse = $dataTable.rows( { selected: false } ).count();
				if(count > 0) {
					control.$count.text(count + ' item' + (count > 1 ? 's' : '') + ' selected');
				}
				if(count == 0) {
					control.$container.removeClass('multi-select');
					$checkboxHeader.removeClass('select-some');
					$checkboxHeader.removeClass('select-all');
				} else if(countFalse == 0) {
					$checkboxHeader.removeClass('select-some');
					$checkboxHeader.addClass('select-all');
				} else {
					$checkboxHeader.removeClass('select-all');
					$checkboxHeader.addClass('select-some');
				}
			});

			$dataTable.on('row-reorder',function(e,diff,edit) {
				sortChanges = new Array();
				// var result = 'Reorder started on row: ' + edit.triggerRow.data()[1] + '\n';
				for(var i=0, ien = diff.length;i<ien;i++) {
					var rowData = $dataTable.row(diff[i].node).data();
					sortChanges[rowData[1]] = diff[i].newData;
					// result += rowData[1] + ' updated to be in position ' + diff[i].newData + ' (was ' + diff[i].oldData + ')\n';
				}
				if(sortChanges.length > 0) {
					$.ajax({
						type: "POST",
						url: fionaUrl + '/debug/sort',
						data: {sort:sort,data:sortChanges},
						dataType: "text",
						timeout:15000
					}).done(function( data ) {
					}).fail(function(jqxhr, textStatus, error) {
						console.error("AJAX failed, status: " + textStatus + ", error: "+error);
					});
				}
				// console.log(sortChanges);
				// console.log('Event result:<br>' + result);
			});

			/** Controls **/
			control.$remove.on('click',function() {
				$dataTable.rows().deselect();
				$checkboxHeader.removeClass('select-some');
				$checkboxHeader.removeClass('select-all');
			});

			control.$delete.on('click',function() {
				var $modal = $('#generic-modal');
				var modalComponent = {
					$title: $modal.find('.modal-title'),
					$body: $modal.find('.modal-body'),
					$yesButton: $modal.find('.modal-yes')
				};
				var count = $dataTable.rows({selected:true}).count();

				if(count > 0) {
					modalComponent.$title.html('Moving <code>' + count + ' item' + (count > 1 ? 's' : '') + '</code> to <code>Trash</code>');

					var html = 'Are you sure you want to move ' + (count == 1 ? 'this item' : 'these items') + ' to Trash?<ul class="trash-items">';
					$dataTable.rows({selected:true}).every(function ( rowIdx, tableLoop, rowLoop ) {
						var data = this.data();
						data = data[3];
						data = data['@data-search'];
						html += '<li class="trash-item">' + data + '</li>';
					});
					html += '</ul>';
					modalComponent.$body.html(html);

					// link = '<?php echo site_url(FIONA_URL_PATH . '/' . strtolower($data['controller']) . '/delete'); ?>/' + id;
					// modalComponent.$yesButton.attr('href',link);
					modalComponent.$yesButton.html('Move to Trash');

					$modal.modal('show');
				}
			});

			control.$filter.on('keyup', function () {
				$dataTable.search(this.value,true,false).draw();

				var word_filter_input = this.value;
				var word_filter = word_filter_input.toLowerCase();
							
				var item = $('.list-item');

				for(var i = 0;i < item.length;i++) {
					var element = $(item[i]);

					var list_item_data = element.find('.list-item-data');

					for(var j = 0;j < list_item_data.length;j++) {
						var list_item_data_inner = $(list_item_data[j]);
						var list_item_data_text = list_item_data_inner.text();
						list_item_data_text = list_item_data_text.replace(/(<span=\"highlight\">|<\/span>)/igm, "");
						list_item_data_inner.text(list_item_data_text);
						var list_item_data_filter = list_item_data_text.toLowerCase();

						if(word_filter.length > 0) {
							var highlight = list_item_data_text.replace(new RegExp("(" + word_filter_input + ")","gim"),"<span class=\"highlight\">$1</span>");
							list_item_data_inner.html(highlight);
						}
					}
				}
			});
			/** End of Controls */

			$checkboxHeader.on('click',function() {
				if($checkboxHeader.hasClass('select-some') || $checkboxHeader.hasClass('select-all')) {
					$checkboxHeader.removeClass('select-all');
					$checkboxHeader.removeClass('select-some');
					$dataTable.rows().deselect();
				} else {
					$checkboxHeader.addClass('select-all');
					$dataTable.rows().select();
				}
			});

			var cobacuy = false;
			var prev = -1;
			var prevAction = 1;
			var direction = 0; //-1 = up, 1 = down, 0 = init

			$('.selection-container').on({
				mousedown: function(event) {
					event.preventDefault();
					var row = $(this).data('row');
					cobacuy = true;
					if(event.shiftKey && prev != -1) {
						// if(prev < row) {
						// 	var from = prev + 1;
						// 	var to = row;
						// } else {
						// 	var from = row;
						// 	var to = prev - 1;
						// }
						// for(var i = from;i <= to;i++) {
						// 	if(prevAction == 0) {
						// 		$dataTable.row(i).deselect();
						// 	} else {
						// 		$dataTable.row(i).select();
						// 	}
						// }
					} else {
						prev = row;
						if($dataTable.row(row,{selected: true}).any()) {
							$dataTable.row(row).deselect();
							prevAction = 0;
						} else {
							$dataTable.row(row).select();
							prevAction = 1;
						}
					}
				},
				mouseenter: function(event) {
					// var row = $(this).data('row');
					// if(cobacuy) {
					// 	if(direction == 0) {
					// 		if(row > prev) {
					// 			direction = 1;
					// 		} else {
					// 			direction = -1;
					// 		}
					// 	}
					// 	if(direction == 1) {
					// 		if(prev > row) {
					// 			$dataTable.row(prev).deselect();
					// 		} else {
					// 			$dataTable.row(row).select();
					// 		}
					// 	} else if(direction == -1) {
					// 		if(prev < row) {
					// 			$dataTable.row(prev).deselect();
					// 		} else {
					// 			$dataTable.row(row).select();
					// 		}
					// 	}
					// 	prev = row;
					// }
				},
				mouseleave: function(event) {
					// var row = $(this).data('row');
					// if(cobacuy) {
					// 	prev = row;
					// }
				}
			});

			// $(document).on({
			// 	mouseup: function() {
			// 		cobacuy = false;
			// 		direction = 0;
			// 	}
			// });

		});
	}

}(jQuery);

$(function () {
	//Textbox Functions
	$('.fiona-input-restrict').fionaRestrict();
	$('.fiona-input-num-separator').fionaAutoThousand();

	//Password Functions
	$('.fiona-password-reset').fionaPassword();

	//Textarea Functions
	if(typeof autosize === 'function') {
		autosize($('.fiona-input-textarea'));
	}

	//Text Editor Functions
	$('.fiona-texteditor').fionaTextEditor();

	//File Functions
	$('.fiona-file').fionaFile();

	//Date Functions
	$('.fiona-date').fionaDate();

	//Switch Functions
	$('.fiona-switch').fionaSwitch();

	//Dropdown Functions
	if(typeof selectpicker === 'function') {
		$('.fiona-dropdown').selectpicker();
	}

	//Map Functions
	$('.fiona-map').fionaMap();

	//Group Functions
	$('.fiona-form-tabs').fionaTabs();

	//Global Functions
	$('.fiona-input-maxlength').fionaMaxLength();
	$('[data-toggle="tooltip"]').tooltip();

	$(".dot17").dotdotdot({
		height: 17,
		watch: "window"
	});

	$('.data-management-body').scroll(function(){
		var scrollTop = $(this).scrollTop();
		if (scrollTop <= 10) {
			$('#controls').removeClass('cast-shadow');
		} else {
			$('#controls').addClass('cast-shadow');
		}
	});

	var $headerSearch = $('.search');
	var $searchPlaceholder = $('.search-placeholder');
	var $search = $('#search');
	var $searchModal = $('#search-modal');

	$searchPlaceholder.on('focus',function() {
		$headerSearch.addClass('selected');
		$searchPlaceholder.blur();
		$searchModal.modal('show');
		$('body').addClass('blur');
	});
	$searchModal.on('shown.bs.modal', function (event) {
		$search.focus();
	});
	$searchModal.on('hide.bs.modal', function (event) {
		$headerSearch.removeClass('selected');
		$('body').removeClass('blur');
	});

	$('.data-table').fionaDataTable();
});

//Login Page Specific Scripts
if($('#login').length > 0) {
	var ajax_login_process = false;

	function login(url) {
		if(!ajax_login_process) {
			ajax_login_process = true;
			$('.login-loading-dots').addClass('animate');
			$('.login-signing-in').addClass('active');
			$('.login-notification').removeClass('active');

			loading_timer = setTimeout(function() {
				$.ajax({
					type: "POST",
					url: url,
					data: $('#fiona_login').serialize(),
					dataType: "json",
					timeout:15000
				}).done(function( data ) {
					if(data.success == 1) {
						window.location.replace(data.url);
					} else {
						$('.login-notification').addClass('active');
						$('.login-notification-message').html(data.message);

						if(data.block == 1) {
							$('.login-block').addClass('active');
							$('.login-form').remove();
						}

						ajax_login_process = false;

						$('.login-loading-dots').removeClass('animate');
						$('.login-signing-in').removeClass('active');

						if($('#login_user').val() == '') $('#login_user').focus();
						else $('#login_pass').focus();
						$('#login_pass').val("");
					}
				}).fail(function(jqxhr, textStatus, error) {
					$('.login-notification').addClass('active');
					$('.login-notification-message').html('Connection time out.');
					ajax_login_process = false;

					$('.login-loading-dots').removeClass('animate');
					$('.login-signing-in').removeClass('active');

					if($('#login_user').val() == '') $('#login_user').focus();
					else $('#login_pass').focus();
					$('#login_pass').val("");
					// $.ajax(this);
					// ajax_login(url);
					console.error("AJAX failed, status: " + textStatus + ", error: "+error);
				});
			},500);
		}
		
		return false;
	}

	//Update 'fiona_expired_last_url' session
	setInterval(function() {
		$.ajax({
			type: "POST",
			url: fiona_url + '/system/ajax_update_expire',
			dataType: "text"
		}).done(function( data ) {
		}).fail(function(jqxhr, textStatus, error) {
			console.error("AJAX failed, status: " + textStatus + ", error: "+error);
		});
	},600000);
}
